<?php
# /modules/repetito/models/Repetito.php

/**
 * Repetito - A Prestashop Module
 * 
 * Vykročte s námi k zodpovědnosti
 * 
 * @author Daniel Bernátek <daniel@bernatek.it>
 * @version 0.0.1
 */

if (!defined('_PS_VERSION_')) exit;

class RepetitoModel extends ObjectModel
{
	public static function installSql()
	{
		// file_put_contents("cs_log", "installSql1\n", FILE_APPEND);

		$results = [];
		
		$sql = "
			CREATE TABLE IF NOT EXISTS `cs_repetito` (
				`id` tinyint unsigned NOT NULL DEFAULT 1,
				`showImageInCart` tinyint unsigned NOT NULL,
				`status` tinyint unsigned NOT NULL,
				`softS` tinyint unsigned NOT NULL,
				`softM` tinyint unsigned NOT NULL,
				`softL` tinyint unsigned NOT NULL,
				`boxS` tinyint unsigned NOT NULL,
				`boxM` tinyint unsigned NOT NULL,
				`fee` float(2) NOT NULL,
				`tax` smallint unsigned NOT NULL,
				`maxWeight` smallint unsigned NOT NULL,
				`country` smallint unsigned NOT NULL,
				PRIMARY KEY (id)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;
		";

		array_push($results, Db::getInstance()->execute($sql));
		// file_put_contents("cs_log", "installSql2\n", FILE_APPEND);

		$sql = "
			CREATE TABLE IF NOT EXISTS `cs_repetito_products` (
				`prodID` int(10) unsigned NOT NULL,
				PRIMARY KEY (prodID)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;
		";

		array_push($results, Db::getInstance()->execute($sql));
		// file_put_contents("cs_log", "installSql3\n", FILE_APPEND);

		$defaultTaxID = 0;
		$defaultCountryID = Configuration::get('PS_COUNTRY_DEFAULT');

		$sql = "
			INSERT IGNORE INTO cs_repetito (id, showImageInCart, status, softS, softM, softL, boxS, boxM, fee, tax, maxWeight, country) VALUES (1, 1, 0, 0, 0, 0, 0, 0, 0, " . $defaultTaxID . ", 0, " . $defaultCountryID . ");
		";

		array_push($results, Db::getInstance()->execute($sql));

		foreach ($results as $result) {
			if (!$result) return false;
		}

		// file_put_contents("cs_log", "installSql7\n", FILE_APPEND);

		return true;
	}

	public static function uninstallSql()
	{
		$results = [];
		
		$sql = "
			DROP TABLE IF EXISTS `cs_repetito`;
		";

		array_push($results, Db::getInstance()->execute($sql));

		$sql = "
			DROP TABLE IF EXISTS `cs_repetito_products`;
		";

		array_push($results, Db::getInstance()->execute($sql));

		foreach ($results as $result) {
			if (!$result) return false;
		}

		return true;
	}

	static function getSettings() {
		$sql = "
			SELECT * FROM cs_repetito WHERE id = 1;
		";

		return Db::getInstance()->executeS($sql);
	}

	// static function getDefaultTax() {

	// 	// First, get all tax rules groups
	// 	$taxRulesGroups = TaxRulesGroup::getTaxRulesGroups(true);

	// 	// Check if there are any tax rules groups
	// 	if (!empty($taxRulesGroups)) {
	// 		foreach ($taxRulesGroups as $group) {
	// 			// Get the ID of the tax rules group
	// 			$groupId = $group['id_tax_rules_group'];

	// 			return $groupId;
	// 		}
	// 	}
	// }

	static function getTaxOptions() {
		// First, get all tax rules groups
		$taxRulesGroups = TaxRulesGroup::getTaxRulesGroups(true);

		$html = "<option value='" . 0 . "'>" . "Bez daně" . "</option>";
		$settings = RepetitoModel::getSettings();

		// Check if there are any tax rules groups
		if (!empty($taxRulesGroups)) {
			foreach ($taxRulesGroups as $group) {
				// Get the ID of the tax rules group
				$groupId = $group['id_tax_rules_group'];

				if ($settings[0]["tax"] == $group['id_tax_rules_group']) {
					$html .= "<option selected value='" . $group['id_tax_rules_group'] . "'>" . $group['name'] . "</option>";
				} else {
					$html .= "<option value='" . $group['id_tax_rules_group'] . "'>" . $group['name'] . "</option>";
				}
			}
		}

		return $html;
	}

	static function getCountryOptions() {
		// Get the current language ID
		$languageId = Context::getContext()->language->id;

		// Get all active countries
		$activeCountries = Country::getCountries($languageId, true); // The second parameter 'true' ensures only active countries are returned

		$html = "";
		$settings = RepetitoModel::getSettings();

		// Iterate through the active countries and print their IDs and names
		foreach ($activeCountries as $country) {
			if ($settings[0]["country"] == $country['id_country']) {
				$html .= "<option selected value='" . $country['id_country'] . "'>" . $country['name'] . "</option>";
			} else {
				$html .= "<option value='" . $country['id_country'] . "'>" . $country['name'] . "</option>";
			}
		}

		return $html;
	}

	static function saveSettings($data) {
		$sql = "
			UPDATE cs_repetito SET showImageInCart = " . (!empty($data["showImageInCart"]) ? 1 : 0) . ", status = " . (!empty($data["status"]) ? 1 : 0) . ", softS = " . (!empty($data["softS"]) ? 1 : 0) . ", softM = " . (!empty($data["softM"]) ? 1 : 0) . ", softL = " . (!empty($data["softL"]) ? 1 : 0) . ", boxS = " . (!empty($data["boxS"]) ? 1 : 0) . ", boxM = " . (!empty($data["boxM"]) ? 1 : 0) . ";
		";

		return array(Db::getInstance()->execute($sql), Db::getInstance()->getMsgError());
	}

	static function saveFees($data) {
		$sql = "
			UPDATE cs_repetito SET fee = " . $data["fee"] . ", tax = " . $data["tax"] . ", maxWeight = " . $data["maxWeight"] . ", country = " . $data["country"] . ";
		";

		return array(Db::getInstance()->execute($sql), Db::getInstance()->getMsgError());
	}

	static function useRepetito($cartID) {

		die(Db::getInstance()->execute("INSERT IGNORE INTO cs_repetito_products(prodID) VALUES(" . $cartID . ");"));
	}

	static function doNotUseRepetito($cartID) {

		die(Db::getInstance()->execute("DELETE FROM cs_repetito_products WHERE prodID = " . $cartID . ";"));
	}

	static function doWeUseRepetito($cartID) {
		return Db::getInstance()->executeS("SELECT prodID FROM cs_repetito_products WHERE prodID = " . $cartID . ";");
	}

	static function calculateFeeWithTax($fee, $taxID, $countryID) {
		
		$address = new Address();
		$address->id_country = $countryID;

		$taxManager = TaxManagerFactory::getManager($address, $taxID);
		$taxCalculator = $taxManager->getTaxCalculator();

		$taxRate = $taxCalculator->getTotalRate();

		return $fee + ($fee / 100) * $taxRate;
	}
	
}