<?php
# /modules/repetito/repetito.php

/**
 * Repetito - A Prestashop Module
 * 
 * Vykročte s námi k zodpovědnosti
 * 
 * @author Daniel Bernátek <daniel@bernatek.it>
 * @version 0.0.1
 */

if (!defined('_PS_VERSION_')) exit;
require_once(__DIR__ . '/models/Repetito.php');

class Repetito extends Module
{
	public function __construct()
	{
		$this->initializeModule();
	}

	public function install()
	{
		return
			parent::install()
			&& $this->installTab()
			&& $this->registerHook('displayHeader')
			&& $this->registerHook('displaybackOfficeHeader')
			// && $this->registerHook('actionCartSummary')
			// && $this->registerHook('actionValidateOrder')
			// && $this->registerHook('displayCheckoutSubtotalDetails')
			// && $this->registerHook('actionCartSave')
			&& $this->registerHook('displayAfterCarrier')
			// && $this->registerHook('displayShoppingCartFooter')
			// && $this->registerHook('actionEmailAddAfterContent')
			&& $this->registerHook('displayAdminOrder')
			// && $this->registerHook('displayAdminOrderTabContent') // rozdílný hook pro různé verze
			// && $this->registerHook('displayOrderConfirmation')
			// && $this->registerHook('displayOrderConfirmation1')
			// && $this->registerHook('displayOrderConfirmation2')
			&& $this->registerHook('actionValidateStepComplete')
			&& RepetitoModel::installSql()
		;
	}

	public function uninstall()
	{
		return
			parent::uninstall()
			&& $this->uninstallTab()
			&& $this->unregisterHook('displayHeader')
			&& $this->unregisterHook('displaybackOfficeHeader')
			// && $this->unregisterHook('actionCartSummary')
			// && $this->unregisterHook('actionValidateOrder')
			// && $this->unregisterHook('displayCheckoutSubtotalDetails')
			// && $this->unregisterHook('actionCartSave')
			&& $this->unregisterHook('displayAfterCarrier')
			// && $this->unregisterHook('displayShoppingCartFooter')
			// && $this->unregisterHook('actionEmailAddAfterContent')
			&& $this->unregisterHook('displayAdminOrder')
			// && $this->unregisterHook('displayAdminOrderTabContent') // rozdílný hook pro různé verze
			// && $this->unregisterHook('displayOrderConfirmation')
			// && $this->unregisterHook('displayOrderConfirmation1')
			// && $this->unregisterHook('displayOrderConfirmation2')
			&& $this->unregisterHook('actionValidateStepComplete')
			&& RepetitoModel::uninstallSql()
		;
	}
	
	public function hookDisplayHeader()
	{
		$this->context->controller->addCSS($this->_path . 'views/css/repetito.css', 'all');
		$this->context->controller->addJS($this->_path . 'views/js/notify.min.js');
		$this->context->controller->addJS($this->_path . 'views/js/repetito_fo.js');
	}

	public function hookDisplayBackOfficeHeader()
	{
		$this->context->controller->addJS($this->_path . 'views/js/notify.min.js');
		$this->context->controller->addJS($this->_path . 'views/js/repetito_bo.js');
		$this->context->controller->addCSS($this->_path . 'views/css/repetito_bo.css');
		Media::addJsDef([
			'repetito_token' => $this->context->controller->token,
		]);
	}

	public function hookDisplayAdminOrder($params)
	{
		$order = new Order($params["id_order"]);

		// Check if the order object is loaded successfully
		if (Validate::isLoadedObject($order)) {
			// Get the cart ID from the order
			$cartId = $order->id_cart;

			if (RepetitoModel::doWeUseRepetito($cartId)) {
				$this->context->smarty->assign([
					'logoPath' => __PS_BASE_URI__ . 'modules/repetito/views/img/repetito.png'
				]);
		
				return $this->display(__FILE__, 'views/templates/hook/adminOrderList.tpl');
			}
		} 

	}

	public function hookDisplayAfterCarrier($params)
	{
		$settings = RepetitoModel::getSettings();

		$cart = Context::getContext()->cart;
		$totalWeight = $cart->getTotalWeight();

		$packages = array(
			"softS" => [$settings[0]["softS"],35,25,10],
			"softM" => [$settings[0]["softM"],50,35,16],
			"softL" => [$settings[0]["softL"],50,35,22],
			"boxS" => [$settings[0]["boxS"],21,11,9],
			"boxM" => [$settings[0]["boxM"],28,21,11]
		);

		// Retrieve products in the cart
		$products = $cart->getProducts();

		// Initialize variables to store the maximum dimensions
		$maxWidth = 0;
		$maxHeight = 0;
		$maxDepth = 0;

		// Iterate through the products to find the maximum dimensions
		foreach ($products as $product) {
			// Product dimensions are accessed via the product array
			$width = $product['width'];
			$height = $product['height'];
			$depth = $product['depth'];

			// Compare and store the maximum dimensions
			if ($width > $maxWidth) {
				$maxWidth = $width;
			}
			if ($height > $maxHeight) {
				$maxHeight = $height;
			}
			if ($depth > $maxDepth) {
				$maxDepth = $depth;
			}
		}

		$canBePackaged = false;
		
		foreach ($packages as $package) {
			// Check if the package is active
			if ($package[0] == 1) {
				// Compare product dimensions with package dimensions
				if ($maxWidth <= $package[1] && $maxHeight <= $package[2] && $maxDepth <= $package[3]) {
					$canBePackaged = true; // The product can be packaged in this package
				}
			}
		}

		$deliveryAddressId = $params["cart"]->id_address_delivery;
		$address = new Address($deliveryAddressId);
		$countryId = $address->id_country;

		if ($settings[0]["status"] && $totalWeight <= $settings[0]["maxWeight"] && $canBePackaged && $countryId == $settings[0]["country"]) {
			// Get the default currency object
			$defaultCurrency = Currency::getDefaultCurrency();

			// Get the currency sign from the default currency object
			$currencySign = $defaultCurrency->sign;

			$this->context->smarty->assign([
				'logoPath' => __PS_BASE_URI__ . 'modules/repetito/views/img/repetito.png',
				'fee' => RepetitoModel::calculateFeeWithTax($settings[0]["fee"], $settings[0]["tax"], $settings[0]["country"]) . ' ' . $currencySign,
				'cartID' => $params["cart"]->id,
				'useRepetito' => RepetitoModel::doWeUseRepetito($params["cart"]->id),
				'showImage' => $settings[0]["showImageInCart"]
			]);

			return $this->display(__FILE__, 'views/templates/hook/checkbox.tpl');
		}
	}

	public function hookActionValidateStepComplete($params)
	{
		
	}
		
	/** Module configuration page */
	public function getContent()
	{
		$adminLink = $this->context->link->getAdminLink('AdminRepetito', true);
    	Tools::redirectAdmin($adminLink);
		// return 'Pro konfiguraci tohoto modulu využijte prosím záložku v levém panelu administrace!';
	}

	/** Initialize the module declaration */
	private function initializeModule()
	{
		$this->name = 'repetito';
		$this->tab = 'front_office_features';
		$this->version = '1.0.1';
		$this->author = 'Daniel Bernátek';
		$this->need_instance = 1;
		$this->ps_versions_compliancy = [
			'min' => '1.7',
			'max' => _PS_VERSION_,
		];
		$this->bootstrap = true;
		
		parent::__construct();

		$this->displayName = 'Repetito';
		$this->description = 'Vykročte s námi k zodpovědnosti.';
		$this->confirmUninstall = 'Určitě chcete tento modul odinstalovat?';
	}

	/** Install module tab, to your admin controller */
	private function installTab()
	{
		// file_put_contents("cs_log", "installTab\n", FILE_APPEND);
		$languages = Language::getLanguages();
		
		$tab = new Tab();
		$tab->class_name = 'AdminRepetito';
		$tab->module = $this->name;
		$tab->id_parent = (int)Tab::getIdFromClassName('DEFAULT');

		foreach ($languages as $lang) {
			$tab->name[$lang['id_lang']] = 'Repetito';
		}

		try {
			$tab->save();
		} catch (Exception $e) {
			return false;
		}

		return true;
	}

	/** Uninstall module tab */
	private function uninstallTab()
	{
		$tab = (int)Tab::getIdFromClassName('AdminRepetito');

		if ($tab) {
			$mainTab = new Tab($tab);

			try {
				$mainTab->delete();
			} catch (Exception $e) {
				echo $e->getMessage();

				return false;
			}
		}

		return true;
	}
}

